Unit GameStat;

Interface

Uses Keyboard,G320,AtariDbg,AVI;

Const
  BI_RGB = 0;

Type
  RGB = Record
    R,G,B: Byte;
  End;
  PPalette   = ^TPalette;
  TPalette   = Array[0..127] Of RGB;

Procedure GameStatus(Width,Height: Word; Var Difficulty1,Difficulty2: LongInt; Var AutoCenter: Boolean);
Procedure ScreenCapture(Width,Height,Palette: Word; Pal0,Pal1: PPalette);
Function  EnterStringPopup(Width,Height: Word; Title,Prompt,OldStr: String): String;
Function  CreateFile(Width,Height: Word; Var F: File; St: String): Boolean;
Function  CanOpenFile(Width,Height: Word; St: String): Boolean;
Procedure SetPalette(Var Pal: TPalette);
Function  FileDialog(Width,Height: Word; Const Title,Filter: String): String;

Implementation

Uses Time,SmalFont,VGAGUI;

Var
  OldKey : Array[0..255] of Byte;
  Saved  : Integer;

Procedure SaveKeys;
Begin
  If Saved = 0 Then
  Begin
    Move(Key,OldKey,SizeOf(OldKey));
    ExitKeyboard;
  End;
  Inc(Saved);
End; { SaveKeys }

Procedure RestoreKeys;
Begin
  If Saved > 0 Then
  Begin
    Dec(Saved);
    If Saved = 0 Then
    Begin
      InitKeyboard;
      Move(OldKey,Key,SizeOf(Key));
    End;
  End;
End; { RestoreKeys }

Procedure SetPalette(Var Pal: TPalette);
Var I: Integer;
Begin
  Port[$3C8] := 0;
  For I := 0 To 127 Do
  Begin
    Port[$3C9] := Pal[I].B Shr 2;
    Port[$3C9] := Pal[I].G Shr 2;
    Port[$3C9] := Pal[I].R Shr 2;

    { Duplicate the colors here to avoid having to worry about bit 0 }

    Port[$3C9] := Pal[I].B Shr 2;
    Port[$3C9] := Pal[I].G Shr 2;
    Port[$3C9] := Pal[I].R Shr 2;
  End; { For I }
End; { SetPalette }

Procedure SaveScreen(Width,BufSize: Word; Buf: Pointer);
Begin
  If Width = 160 Then Move(Mem[$A000:0],Buf^,BufSize)
  Else
  Begin
    Asm
      CLD
      PUSH  DS
      MOV   CX,WORD PTR BufSize
      LES   DI,DWORD PTR Buf
      MOV   AX,0A000h
      MOV   DS,AX
      SUB   SI,SI
@L1:
      MOV   AL,BYTE PTR [SI]
      MOV   BYTE PTR ES:[DI],AL
      ADD   SI,2
      INC   DI
      DEC   CX
      JNZ   @L1
      POP   DS
    End; { Asm }
  End;
End; { SaveScreen }


Procedure RestoreScreen(Width,BufSize: Word; Buf: Pointer);
Begin
  If Width = 160 Then Move(Buf^,Mem[$A000:0],BufSize)
  Else
  Begin
    Asm
      CLD
      PUSH  DS
      MOV   CX,WORD PTR BufSize
      MOV   AX,0A000h
      MOV   ES,AX
      SUB   DI,DI
      LDS   SI,DWORD PTR Buf
@L1:
      MOV   AL,BYTE PTR [SI]
      MOV   AH,AL
      MOV   WORD PTR ES:[DI],AX
      INC   SI
      ADD   DI,2
      DEC   CX
      JNZ   @L1
      POP   DS
    End; { Asm }
  End;
End; { RestoreScreen }

Procedure Dim(BufSize: Word); Assembler;
Asm
  MOV   AX,0A000h
  MOV   ES,AX
  MOV   CX,WORD PTR BufSize
  SHR   CX,1
  SUB   DI,DI
@L1:
  MOV   DX,WORD PTR [ES:DI]
  MOV   BX,DX
  AND   BX,0E0Eh
  AND   DX,0F0F0h

  OR    BL,BL
  JZ    @L2
  SHR   BL,2
  CMP   BL,2
  JGE   @L2
  MOV   BL,2
@L2:
  OR    BH,BH
  JZ    @L3
  SHR   BH,2
  CMP   BH,2
  JGE   @L3
  MOV   BH,2
@L3:
  OR    DX,BX
  MOV   WORD PTR [ES:DI],DX
  ADD   DI,2
  LOOP  @L1
End; { Dim }

Procedure DrawWindow(X1,Y1,X2,Y2: Word; BK,C1,C2: Byte; Width: Word);
Const BorderWidth = 5;
Var
  X,Y   : Word;
  Addr1 : Word;
  Addr2 : Word;
  CInc  : Integer;

Begin
  BK := BK Shl 1;
  C1 := C1 Shl 1;
  C2 := C2 Shl 1;

  { Draw top and bottom borders }

  For Y := 0 To BorderWidth - 1 Do
  Begin
    Addr1 := Width * (Y1 + Y);
    Addr2 := Width * (Y2 + Y - BorderWidth + 1);
    CInc  := 2 * ((BorderWidth Div 2) - Abs((BorderWidth Div 2) - Integer(Y)));
    For X := X1 + 1 To X2 - 1 Do
    Begin
      If ((X + Y + Y1) And 2) <> 0
       Then Mem[$A000:Addr1 + X] := C1 + CInc
       Else Mem[$A000:Addr1 + X] := C2 + CInc;
      If ((X + Y + Y2 - BorderWidth + 1) And 2) <> 0
       Then Mem[$A000:Addr2 + X] := C1 + CInc
       Else Mem[$A000:Addr2 + X] := C2 + CInc;
    End; { For X }
  End; { For Y }

  { Draw left and right borders }

  For Y := Y1 + 1 To Y2 - 1 Do
  Begin
    Addr1 := Width * Y;
    For X := X1 To X1 + BorderWidth - 1 Do
    Begin
      CInc  := 2 * ((BorderWidth Div 2) - Abs((BorderWidth Div 2) - Integer(X - X1)));
      If ((X + Y) And 2) <> 0
       Then Mem[$A000:Addr1 + X] := C1 + CInc
       Else Mem[$A000:Addr1 + X] := C2 + CInc;
    End; { For X }
    For X := X2 - BorderWidth + 1 To X2 Do
    Begin
      CInc  := 2 * ((BorderWidth Div 2) - Abs((BorderWidth Div 2) - Integer(X - (X2 - BorderWidth + 1))));
      If ((X + Y) And 2) <> 0
       Then Mem[$A000:Addr1 + X] := C1 + CInc
       Else Mem[$A000:Addr1 + X] := C2 + CInc;
    End; { For X }
  End; { For Y }

  { Draw center area }

  For Y := Y1 + BorderWidth To Y2 - BorderWidth Do
   FillChar(Mem[$A000:Width * Y + X1 + BorderWidth],X2 - X1 - 2 * BorderWidth + 1,BK);
End; { DrawWindow }

Procedure PutChar(X,Y: Word; C: Char; Color,Width: Word); Assembler;
Asm
  MOV   AX,WORD PTR Y
  MUL   WORD PTR Width
  MOV   DI,AX
  ADD   DI,WORD PTR X
  PUSH  DS
  LDS   SI,DWORD PTR FontPtr
  MOV   AL,BYTE PTR C
  SUB   AH,AH
  SHL   AX,3
  ADD   SI,AX
  MOV   AX,0A000h
  MOV   ES,AX
  MOV   AL,8
  ADD   DI,7
  MOV   DX,WORD PTR Color
@Loop:
  MOV   BH,BYTE PTR [SI]
  MOV   CX,8
  MOV   BL,1
@L1:
  TEST  BH,BL
  JZ    @L2
  MOV   BYTE PTR [ES:DI],DL
  DEC   DI
  ADD   BL,BL
  LOOP  @L1
  ADD   DI,WORD PTR Width
  INC   SI
  ADD   DI,8
  DEC   AL
  JNZ   @Loop
  POP   DS
  JMP   @L3
@L2:
  MOV   BYTE PTR [ES:DI],DH
  DEC   DI
  ADD   BL,BL
  LOOP  @L1
  ADD   DI,WORD PTR Width
  INC   SI
  ADD   DI,8
  DEC   AL
  JNZ   @Loop
  POP   DS
@L3:
End; { PutChar }

Procedure WriteOut(X,Y: Word; St: String; Color,Width: Word);
Var I: Byte;
Begin
  If St = '' Then Exit;
  For I := 0 To Length(St) - 1 Do PutChar(X + (I * 8),Y,St[I + 1],Color,Width);
End; { WriteOut }

Procedure GameStatus(Width,Height: Word; Var Difficulty1,Difficulty2: LongInt; Var AutoCenter: Boolean);
Var
  Buf     : Pointer;
  BufSize : LongInt;

  Procedure DrawDiff(X,Y: Word; Diff: Boolean; Width: Word);
  Begin
    If Diff Then WriteOut(X,Y,'A',$089F,Width) Else WriteOut(X,Y,'B',$089F,Width);
  End; { DrawDiff }

  Procedure DrawYN(X,Y: Word; Diff: Boolean; Width: Word);
  Begin
    If Diff Then WriteOut(X,Y,'Y',$089F,Width) Else WriteOut(X,Y,'N',$089F,Width);
  End; { DrawDiff }

Begin
  BufSize := Width * Height;
  If Width = 320 Then BufSize := BufSize Shr 1;
  GetMem(Buf,BufSize);
  SaveScreen(Width,BufSize,Buf);
  Key[kEsc] := 0;
  If Width = 320
   Then Dim(BufSize * 2)
   Else Dim(BufSize);
  InitGUI(Width,Height,False,True);
  If Width = 320 Then
  Begin
    DrawWindow(100,15,220,185,4,5,28,Width);
    WriteOut(108,22,'PCAE 2.5',$081E,Width);
    WriteSmOut(108,32, 'Esc - Exit game',$0800);
    WriteSmOut(108,42, 'F1  - Help',$0800);
    WriteSmOut(108,52, 'F2  - Reset',$0800);
    WriteSmOut(108,62, 'F3  - Toggle PAL',$0800);
    WriteSmOut(108,72, 'F4  - Select',$0800);
    WriteSmOut(108,82, 'F5  - Color/BW',$0800);
    WriteSmOut(108,92, 'F6  - P1 diff.',$0886);
    WriteSmOut(108,102,'F7  - P2 diff.',$0886);
    WriteSmOut(108,112,'F8  - Save info',$0800);
    WriteSmOut(108,122,'F9  - Sc.capture',$0800);
    WriteSmOut(108,132,'F10 - Autocenter',$0886);
    WriteSmOut(108,142,'F11 - Save game',$0800);
    WriteSmOut(108,152,'F12 - Load game',$0800);
    WriteSmOut(108,162,'-/= - Shift dn/up',$0800);
    WriteSmOut(108,172,'BKSP- Save to AVI',$0800);
    DrawDiff(208,92,((Difficulty1 And $40) <> 0),Width);
    DrawDiff(208,102,((Difficulty2 And $80) <> 0),Width);
    DrawYN(208,132,AutoCenter,Width);
  End
  Else
  Begin
    DrawWindow(20,15,140,185,4,5,28,Width);
    WriteOut(28,22,'PCAE 2.5',$081E,Width);
    WriteSmOut(28,32, 'Esc - Exit game',$0800);
    WriteSmOut(28,42, 'F1  - Help',$0800);
    WriteSmOut(28,52, 'F2  - Reset',$0800);
    WriteSmOut(28,62, 'F3  - Toggle PAL',$0800);
    WriteSmOut(28,72, 'F4  - Select',$0800);
    WriteSmOut(28,82, 'F5  - Color/BW',$0800);
    WriteSmOut(28,92, 'F6  - P1 diff.',$0886);
    WriteSmOut(28,102,'F7  - P2 diff.',$0886);
    WriteSmOut(28,112,'F8  - Save info',$0800);
    WriteSmOut(28,122,'F9  - Sc.Capture',$0800);
    WriteSmOut(28,132,'F10 - Autocenter',$0886);
    WriteSmOut(28,142,'F11 - Save game',$0800);
    WriteSmOut(28,152,'F12 - Load game',$0800);
    WriteSmOut(28,162,'-/= - Shift dn/up',$0800);
    WriteSmOut(28,172,'BKSP- Save to AVI',$0800);
    DrawDiff(128,92, ((Difficulty1 And $40) <> 0),Width);
    DrawDiff(128,102,((Difficulty2 And $80) <> 0),Width);
    DrawYN(128,132,AutoCenter,Width);
  End;
  Repeat
    If Key[kF6] <> 0 Then
    Begin
      Difficulty1 := Difficulty1 Xor $40;
      If Width = 320
       Then DrawDiff(208,92,((Difficulty1 And $40) <> 0),Width)
       Else DrawDiff(128,92,((Difficulty1 And $40) <> 0),Width);
      Key[kF6] := 0;
    End;
    If Key[kF7] <> 0 Then
    Begin
      Difficulty2 := Difficulty2 Xor $80;
      If Width = 320
       Then DrawDiff(208,102,((Difficulty2 And $80) <> 0),Width)
       Else DrawDiff(128,102,((Difficulty2 And $80) <> 0),Width);
      Key[kF7] := 0;
    End;
    If Key[kF10] <> 0 Then
    Begin
      AutoCenter := Not AutoCenter;
      If Width = 320
       Then DrawYN(208,132,AutoCenter,Width)
       Else DrawYN(128,132,AutoCenter,Width);
      Key[kF10] := 0;
    End;
  Until Key[kEsc] <> 0;
  Repeat
  Until Key[kEsc] = 0;
  RestoreScreen(Width,BufSize,Buf);
  FreeMem(Buf,BufSize);
End; { GameStatus }


Function EnterStringPopup(Width,Height: Word; Title,Prompt,OldStr: String): String;
Var
  Buf     : Pointer;
  BufSize : LongInt;
  St      : String;
  F       : File;
  Indent  : Integer;
  App     : PVGAApplication;

Begin
  SaveKeys;
  BufSize := Width * Height;
  If Width = 320 Then BufSize := BufSize Shr 1;
  GetMem(Buf,BufSize);
  SaveScreen(Width,BufSize,Buf);
  If Width = 320
   Then Dim(BufSize * 2)
   Else Dim(BufSize);
  App := New(PVGAApplication,Init(Width,Height,False,False,True));
  EnterStringPopup := Trim(App^.InputBox(Title,Prompt,OldStr));
  Dispose(App,Done);
  RestoreScreen(Width,BufSize,Buf);
  FreeMem(Buf,BufSize);
  RestoreKeys;
End; { EnterStringPopup }


Function CanOpenFile(Width,Height: Word; St: String): Boolean;
Var
  Buf     : Pointer;
  BufSize : LongInt;
  Error   : Boolean;
  App     : PVGAApplication;

Begin
  SaveKeys;
  Error   := False;
  BufSize := Width * Height;
  If Width = 320 Then BufSize := BufSize Shr 1;
  If St = '' Then Error := True;
  If Not FileExists(St) Then Error := True;
  If Error Then
  Begin
    If St <> '' Then
    Begin
      GetMem(Buf,BufSize);
      SaveScreen(Width,BufSize,Buf);
      If Width = 320
       Then Dim(BufSize * 2)
       Else Dim(BufSize);
      App := New(PVGAApplication,Init(Width,Height,False,False,True));
      App^.ShowMessage('Error','Could not open file',[mbOK]);
      Dispose(App,Done);
      RestoreScreen(Width,BufSize,Buf);
      FreeMem(Buf,BufSize);
    End;
    CanOpenFile := False;
  End
  Else CanOpenFile := True;
  RestoreKeys;
End; { CanOpenFile }


Function CreateFile(Width,Height: Word; Var F: File; St: String): Boolean;
Var
  Buf      : Pointer;
  BufSize  : LongInt;
  Error    : Boolean;
  Continue : Boolean;
  App      : PVGAApplication;

Begin
  SaveKeys;
  Error   := False;
  BufSize := Width * Height;
  If Width = 320 Then BufSize := BufSize Shr 1;
  If St = '' Then Error := True;

  Continue := True;
  If (Not Error) And FileExists(St) Then
  Begin
    GetMem(Buf,BufSize);
    SaveScreen(Width,BufSize,Buf);
    If Width = 320
     Then Dim(BufSize * 2)
     Else Dim(BufSize);
    App := New(PVGAApplication,Init(Width,Height,False,False,True));
    Continue := (App^.ShowMessage('Warning: file already exists','Overwrite file ' + St +
                              '?',[mbOK,mbCancel]) = idOK);
    Dispose(App,Done);
    RestoreScreen(Width,BufSize,Buf);
    FreeMem(Buf,BufSize);
  End;
  If Error Then Continue := False;

  If Continue Then
  Begin
    Assign(F,St);
    {$I-}
    ReWrite(F,1);
    {$I+}
    If IOResult <> 0 Then Error := True;
    If Error Then
    Begin
      Repeat
      Until Key[kEsc] = 0;
      If St <> '' Then
      Begin
        GetMem(Buf,BufSize);
        SaveScreen(Width,BufSize,Buf);
        If Width = 320
         Then Dim(BufSize * 2)
         Else Dim(BufSize);
        App := New(PVGAApplication,Init(Width,Height,False,False,True));
        App^.ShowMessage('Error','Could not create file',[mbOK]);
        Dispose(App,Done);
        RestoreScreen(Width,BufSize,Buf);
        FreeMem(Buf,BufSize);
      End;
      CreateFile := False;
    End
    Else CreateFile := True;
  End
  Else CreateFile := False;
  RestoreKeys;
End; { CreateFile }


Procedure ScreenCapture(Width,Height,Palette: Word; Pal0,Pal1: PPalette);
Var
  Buf     : Pointer;
  BufSize : LongInt;
  St      : String;
  F       : File;
  App     : PVGAApplication;

  Procedure MakeBMPFile(Var F: File);
  Var
    BF      : BITMAPFILEHEADER;
    BI      : BITMAPINFOHEADER;
    I       : Integer;
    Q       : RGBQuad;

  Begin
    FillChar(BF,SizeOf(BF),#0);
    FillChar(BI,SizeOf(BI),#0);

    BF.bfType[0] := 'B';
    BF.bfType[1] := 'M';
    BF.bfSize    := SizeOf(BF) + SizeOf(BI) + 256 * SizeOf(RGBQuad) + BufSize;
    BF.bfOffBits := SizeOf(BF) + SizeOf(BI) + 256 * SizeOf(RGBQuad);

    BI.biSize          := 40;
    BI.biWidth         := Width;
    BI.biHeight        := Height;  { Top-down bitmap }
    BI.biPlanes        := 1;
    BI.biBitCount      := 8;
    BI.biCompression   := BI_RGB;
    BI.biSizeImage     := 0;
    BI.biXPelsPerMeter := 2000;
    BI.biYPelsPerMeter := 2000;
    BI.biClrUsed       := 256;
    BI.biClrImportant  := 256;

    BlockWrite(F,BF,SizeOf(BF));
    BlockWrite(F,BI,SizeOf(BI));

    If Palette = 0 Then
    Begin
      For I := 0 To 255 Do
      Begin
        Q.R := Pal0^[I Shr 1].B;
        Q.G := Pal0^[I Shr 1].G;
        Q.B := Pal0^[I Shr 1].R;
        Q.Reserved := 0;
        BlockWrite(F,Q,SizeOf(Q));
      End; { For I }
    End
    Else
    Begin
      For I := 0 To 255 Do
      Begin
        Q.R := Pal1^[I Shr 1].B;
        Q.G := Pal1^[I Shr 1].G;
        Q.B := Pal1^[I Shr 1].R;
        Q.Reserved := 0;
        BlockWrite(F,Q,SizeOf(Q));
      End; { For I }
    End;
    For I := 0 To Height - 1 Do
     BlockWrite(F,Mem[Seg(Buf^):Ofs(Buf^) + (Height - I - 1) * Width],Width);
  End; { MakeBMPFile }

Begin
  SaveKeys;
  BufSize := Width * Height;
  If Width = 320 Then BufSize := BufSize Shr 1;
  GetMem(Buf,BufSize);
  SaveScreen(Width,BufSize,Buf);
  If Width = 320
   Then Dim(BufSize * 2)
   Else Dim(BufSize);
  App := New(PVGAApplication,Init(Width,Height,False,False,True));
  St  := UpperCase(Trim(App^.FileDialog('Save screenshot to BMP','*.*','')));
  Dispose(App,Done);
  If CreateFile(Width,Height,F,St) Then
  Begin
    MakeBMPFile(F);
    Close(F);
  End;
  RestoreScreen(Width,BufSize,Buf);
  FreeMem(Buf,BufSize);
  RestoreKeys;
End; { ScreenCapture }

Function FileDialog(Width,Height: Word; Const Title,Filter: String): String;
Var
  Buf     : Pointer;
  BufSize : LongInt;
  St      : String;
  App     : PVGAApplication;

Begin
  SaveKeys;
  BufSize := Width * Height;
  If Width = 320 Then BufSize := BufSize Shr 1;
  GetMem(Buf,BufSize);
  SaveScreen(Width,BufSize,Buf);
  If Width = 320
   Then Dim(BufSize * 2)
   Else Dim(BufSize);
  App := New(PVGAApplication,Init(Width,Height,False,False,True));
  St  := UpperCase(Trim(App^.FileDialog(Title,Filter,'')));
  Dispose(App,Done);
  RestoreScreen(Width,BufSize,Buf);
  FreeMem(Buf,BufSize);
  RestoreKeys;
  FileDialog := St;
End; { FileDialog }

Begin
  Saved := 0;
End.

// ----------------------------------------------------------------------
// PCAE - PC Atari Emulator - Atari 2600 Emulator for MS-DOS
// Copyright (C) 2000 John Dullea
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
// ----------------------------------------------------------------------

